//
//  NVPolylineAnnotationView.m
//  nvpolyline
//
//  Created by William Lachance on 10-03-31.
//  Inspired by code and ideas from Craig Spitzkoff and Nicolas Neubauer 2009.
//

#import "PolylineAnnotationView.h"

const CGFloat POLYLINE_WIDTH = 4.0;

@interface PolylineInternalAnnotationView : UIView {
	PolylineAnnotationView* _polylineView;
	MKMapView *_mapView;
}

- (id) initWithPolylineView:(PolylineAnnotationView *)polylineView
					mapView:(MKMapView *)mapView;

@end

@implementation PolylineInternalAnnotationView

- (id) initWithPolylineView:(PolylineAnnotationView *)polylineView
					mapView:(MKMapView *)mapView {
    if (self = [super init]) {
        _polylineView = polylineView;
        _mapView = mapView;
        
        self.backgroundColor = [UIColor clearColor];
        self.clipsToBounds = NO;
    }
    
    return self;
}


-(void) drawRect:(CGRect)rect {
    
	PolylineAnnotation* annotation = (PolylineAnnotation*)_polylineView.annotation;
	if (annotation.points && annotation.points.count > 0)
	{
		CGContextRef context = UIGraphicsGetCurrentContext();
		
		CGContextSetStrokeColorWithColor(context, [UIColor blueColor].CGColor);
		CGContextSetRGBFillColor(context, 0.0, 0.0, 1.0, 1.0);
		CGContextSetAlpha(context, .5);
		
		CGContextSetLineWidth(context, POLYLINE_WIDTH);
		
		for (int i = 0; i < annotation.points.count; i++) {
			CLLocation* location = [annotation.points objectAtIndex:i];
			CGPoint point = [_mapView convertCoordinate:location.coordinate toPointToView:self];
			
			if (i == 0)
				CGContextMoveToPoint(context, point.x, point.y);
			else
				CGContextAddLineToPoint(context, point.x, point.y);
		}
		
		CGContextStrokePath(context);
	}
}


@end


@implementation PolylineAnnotationView

- (id)initWithAnnotation:(PolylineAnnotation *)annotation
				 mapView:(MKMapView *)mapView {
    if (self = [super init]) {
        self.annotation = annotation;
		
		_mapView = mapView;
		
		self.backgroundColor = [UIColor clearColor];
		self.clipsToBounds = NO;
		self.frame = CGRectMake(0.0, 0.0, _mapView.frame.size.width, _mapView.frame.size.height);
        
		_internalView = [[PolylineInternalAnnotationView alloc] initWithPolylineView:self mapView:_mapView];
		[self addSubview:_internalView];
    }
    return self;
}

-(void) regionChanged {
	// move the internal route view.
	
	PolylineAnnotation* annotation = (PolylineAnnotation*)self.annotation;
	CGPoint minpt, maxpt;
	for (int i = 0; i < annotation.points.count; i++)
	{
		CLLocation* location = [annotation.points objectAtIndex:i];
		CGPoint point = [_mapView convertCoordinate:location.coordinate toPointToView:_mapView];
		if (point.x < minpt.x || i == 0)
			minpt.x = point.x;
		if (point.y < minpt.y || i == 0)
			minpt.y = point.y;
		if (point.x > maxpt.x || i == 0)
			maxpt.x = point.x;
		if (point.y > maxpt.y || i == 0)
			maxpt.y = point.y;
	}
	
	CGFloat w = maxpt.x - minpt.x + (2*POLYLINE_WIDTH);
	CGFloat h = maxpt.y - minpt.y + (2*POLYLINE_WIDTH);
	
	_internalView.frame = CGRectMake(minpt.x - POLYLINE_WIDTH, minpt.y - POLYLINE_WIDTH,
									 w, h);
	[_internalView setNeedsDisplay];
}

- (CGPoint) centerOffset {
	// HACK: use the method to get the centerOffset (called by the main mapview)
	// to reposition our annotation subview in response to zoom and motion
	// events
	[self regionChanged];
	return [super centerOffset];
}

- (void) setCenterOffset:(CGPoint) centerOffset {
	[super setCenterOffset:centerOffset];
}



@end
